@@ -11,7 +11,6 @@ import copy
 import numpy as np
 import pandas as pd
 from datetime import timedelta
-import random
 
 from grid2op.dtypes import dt_float, dt_bool
 from grid2op.Exceptions import EnvError, IncorrectNumberOfLoads, IncorrectNumberOfLines, IncorrectNumberOfGenerators
@@ -113,10 +112,14 @@ class GridStateFromFileWithForecasts(GridStateFromFile):
         prod_v_iter = self._get_data("prod_v_forecasted")
         hazards = None  # no hazards in forecast
 
+        nrows = None
+        if self.max_iter > 0:
+            nrows = self.max_iter + 1
         read_compressed = self._get_fileext("maintenance_forecasted")
         if read_compressed is not None:
             maintenance = pd.read_csv(os.path.join(self.path, "maintenance_forecasted{}".format(read_compressed)),
-                                      sep=self.sep)
+                                      sep=self.sep,
+                                      nrows=nrows)
         else:
             maintenance = None
 
