@@ -8,8 +8,7 @@
 
 import os
 import numpy as np
-from datetime import timedelta
-import pdb
+from datetime import timedelta, datetime
 
 from grid2op.dtypes import dt_int
 from grid2op.Exceptions import *
@@ -54,10 +53,12 @@ class Multifolder(GridValue):
     """
     def __init__(self, path,
                  time_interval=timedelta(minutes=5),
+                 start_datetime=datetime(year=2019, month=1, day=1),
                  gridvalueClass=GridStateFromFile,
                  sep=";", max_iter=-1,
                  chunk_size=None):
-        GridValue.__init__(self, time_interval=time_interval, max_iter=max_iter, chunk_size=chunk_size)
+        GridValue.__init__(self, time_interval=time_interval, max_iter=max_iter, chunk_size=chunk_size,
+                           start_datetime=start_datetime)
         self.gridvalueClass = gridvalueClass
         self.data = None
         self.path = os.path.abspath(path)
@@ -70,7 +71,6 @@ class Multifolder(GridValue):
         except FileNotFoundError:
             raise ChronicsError("Path \"{}\" doesn't exists.".format(self.path)) from None
 
-
         if len(self.subpaths) == 0:
             raise ChronicsNotFoundError("Not chronics are found in \"{}\". Make sure there are at least "
                                         "1 chronics folder there.".format(self.path))
