@@ -23,8 +23,6 @@ from grid2op.dtypes import dt_int, dt_float, dt_bool
 from grid2op.Exceptions import *
 from grid2op.Space.space_utils import extract_from_dict, save_to_dict
 
-import pdb
-
 
 # TODO better random stuff when random observation (seed in argument is really weird)
 
@@ -560,7 +558,10 @@ class GridObjects:
         """
         tmp = getattr(self, attr_nm)
         if isinstance(tmp, (dt_bool, dt_int, dt_float)):
-            setattr(self, attr_nm, vect)
+            if isinstance(vect, np.ndarray):
+                setattr(self, attr_nm, vect[0])
+            else:
+                setattr(self, attr_nm, vect)
         else:
             tmp[:] = vect
 
