@@ -14,7 +14,7 @@ import pdb
 import warnings
 
 import grid2op
-from grid2op.tests.helper_path_test import HelperTests, PATH_DATA_TEST_PP
+from grid2op.tests.helper_path_test import HelperTests, PATH_DATA_TEST_PP, PATH_DATA_TEST
 from grid2op.Action import ActionSpace, CompleteAction
 from grid2op.Backend import PandaPowerBackend
 from grid2op.Parameters import Parameters
@@ -26,9 +26,18 @@ from grid2op.MakeEnv import make
 from grid2op.Rules import AlwaysLegal
 from grid2op.Action._BackendAction import _BackendAction
 
+PATH_DATA_TEST_INIT = PATH_DATA_TEST
 PATH_DATA_TEST = PATH_DATA_TEST_PP
 
 
+class TestNames(unittest.TestCase):
+    def test_properNames(self):
+        with make(os.path.join(PATH_DATA_TEST_INIT, "5bus_example_diff_name")) as env:
+            obs = env.reset()
+            assert np.all(obs.name_load == ["tutu", "toto", "tata"])
+            assert np.all(env.name_load == ["tutu", "toto", "tata"])
+
+
 class TestLoadingCase(unittest.TestCase):
     def setUp(self):
         self.tolvect = 1e-2
